import click
import requests
from connectifycli.graph.graph import Graph
from connectifycli.utils.json import convert_to_graph

@click.group()
def main():
    pass

@main.command()
@click.argument("hash")
def run(hash):
    # response = requests.post("", data={})
    # if(response.status_code != 200):
    #     exit("Error retrieving graph")
    click.echo("Running graph {hash}...")
        
    # graph: Graph = convert_to_graph(response.json())
    # run code to for graphs
    


    



class ModelInfo:
    def __init__(self, id: str, source: str, input: dict[str, str] = {},
                 output: dict[str, str] = {}) -> None:
        self.id = id
        
        if(source == "hf_model"):
            self.repo_name = id
        else:
            self.repo_name = ""
            
        self.source = source
        self.input = input
        self.output = output
        
    @property
    def source(self):
        return self._source
    
    @source.setter
    def source(self, source):
        if(source != "hf_model" and source != "custom"):
            raise Exception(f"Model of type {source} not supported. Supported " +
                            "model types: hf_model, custom")
        self._source = source
        
    def __repr__(self):
        return self.id
            
        
        
        


from connectifycli.graph.graph import Graph
from connectifycli.graph.model_info import ModelInfo

# input: graph json from database
# output: Graph object filled with ModelInfo
def convert_to_graph(input_json):
    
    if(input_json.get('graph') is None):
        raise Exception("No graph provided in json")
    
    input_graph: dict = input_json.get('graph')
    if(input_graph == None):
        print("Json provided improper format")
    
    input_nodes: list[dict] = input_graph.get('nodes')
    input_edges: list[dict] = input_graph.get('edges')
    
    if(input_nodes == None or input_edges == None):
        print("Json provided has no nodes or edges")
        
    graph: Graph = Graph()
        
    for node in input_nodes:
        source: str = "hf_model"
        id: str = node.get("repoId")
        
        graph.add_node(ModelInfo(id, source, {}, {}))
        
    for edge in input_edges:
        source_id: str = edge["sourceNode"]["repoId"]
        dest_id: str = edge["targetNode"]["repoId"]
        
        graph.add_edge(source_id, dest_id)
    
    return graph
        


from typing import Optional
from connectifycli.graph.model_info import ModelInfo
from collections import deque

class Graph:

    def __init__(self) -> None:
        self.nodes: dict[ModelInfo, set[ModelInfo]] = {}
        
    @property
    def models(self):
        return self.nodes.keys()
    
    @property
    def edges(self) -> dict[ModelInfo, set]:
        return self.nodes
    
    def add_node(self, model: ModelInfo) -> None:
        if(model in self.nodes.keys()):
            raise Exception(f"Attempting to add duplicate model {model}")
        for model_info in self.nodes.keys():
            if(model.id == model_info.id):
                raise Exception(f"Attempting to add duplicate model ID {model.id}")
            
        self.nodes[model] = set()
        
    def add_edge(self, source: str, dest: str) -> None:
        if(source == dest):
            raise Exception("Attempting to add edge to itself")
        source_model: Optional[ModelInfo] = None
        dest_model: Optional[ModelInfo] = None
        for i, model in enumerate(self.nodes.keys()):
            if(model.id == source):
                source_model = model
            elif(model.id == dest):
                dest_model = model
        
        if(source_model is None):
            raise Exception(f"{source} node not found in graph")
        if(dest_model is None):
            raise Exception(f"{dest} node not found in graph")
        
        if(dest_model in self.nodes[source_model]):
            raise Exception(f"Adding {source} to {dest} edge to graph, which" +
                            " already exists")           
        self.nodes[source_model].add(dest_model)
        
    def __str__(self) -> str:
        string: str = ""
        for i, model in enumerate(self.nodes.keys()):
            string += f"{model.id}:"
            for dest in self.nodes[model]:
                string += dest.id
            string += "\n"
            
        return string.rstrip()
    
    def get_model_info(self, id: str) -> Optional[ModelInfo]:
        for model_info in self.nodes.keys():
            if id == model_info.id:
                return model_info
        return None
    
    def linear_path(self) -> list[ModelInfo]:
        stack: deque = deque()
        visited: dict[ModelInfo, int] = {}
        
        for model_info in self.nodes.keys():
            visited[model_info] = 0
            
        for model_info in self.nodes.keys():
            if not visited[model_info]:
                self.dfs(model_info, visited, stack)
            
        path: list[ModelInfo] = list(stack)
        path.reverse()
        return path
        
    def dfs(self, model: ModelInfo, visited: dict[ModelInfo, int], stack: deque):
        visited[model] = -1
        
        for dest_model in self.nodes[model]:
            if visited[dest_model] == -1:
                raise Exception("Graph is not acyclic. Linear path cannot be found.")
            if visited[dest_model] == 0:
                self.dfs(dest_model, visited, stack)
                
        visited[model] = 1
        stack.append(model)
        
    
        
    

        


from setuptools import setup, find_packages

setup(
    name="connectifyai",
    version="0.1.1",
    description="A CLI to run ConnectifyAI graphs",
    url="https://connectify-ai.vercel.app",
    author="ConnectifyAI",
    author_email="connectifyai2@gmail.com",
    classifiers=[
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3 :: Only"
    ],
    install_requires=[
        'Click',
        'orjson',
        'transformers',
        'datasets',
        'requests' 
    ],
    packages=find_packages(),
    python_requires=">=3.8",
    entry_points={
        'console_scripts': [
            "connect = connectifycli.cli:main",
        ],
    },
)
